from tests.tests.unit.register import *
from tests.tests.unit.managers import *
from tests.tests.unit.models import *
from tests.tests.unit.admin import *
from tests.tests.unit.diff import *
from tests.tests.unit.forms import *
from tests.tests.unit.moderator import *
from tests.tests.regression import *
from tests.tests.acceptance.exclude import *
from tests.tests.acceptance.auto_discover import *
